"""
Lovense Socket API event names.
"""

BASICAPI_GET_QRCODE_TS = "basicapi_get_qrcode_ts"
BASICAPI_GET_QRCODE_TC = "basicapi_get_qrcode_tc"
BASICAPI_UPDATE_DEVICE_INFO_TC = "basicapi_update_device_info_tc"
BASICAPI_UPDATE_APP_ONLINE_TC = "basicapi_update_app_online_tc"
BASICAPI_UPDATE_APP_STATUS_TC = "basicapi_update_app_status_tc"
BASICAPI_SEND_TOY_COMMAND_TS = "basicapi_send_toy_command_ts"
